levelbar .strength-weak {
  background-color: @destructive_bg_color;
}

levelbar .strength-low {
  background-color: @warning_bg_color;
}

levelbar .strength-medium,
levelbar .strength-good,
levelbar .strength-high {
  background-color: @success_bg_color;
}
