gettext "Force live user mode"
gettext "Force OEM mode"
gettext "Try or Install?"
gettext "You can try a temporary version of \${NAME} without making changes to the computer. If you decide to install later, just launch the installer from the Activities view.

        Alternatively, choose install to launch the installer."
gettext "Install to Disk…"
gettext "Try \${NAME}"
gettext "Install \${PRETTY_NAME}"
