.cutout-button {
  background-color: @window_bg_color;
  border-radius: 9999px;
  padding: 2px;
}

/* This is used for remove_avatar_button */
.destructive-image-button {
  color: @destructive_color;
}
